library open_container_transition;

import 'package:flutter/material.dart';


class OpenContainerTransition extends StatefulWidget {
  final Widget collapsedChild;
  final Widget expandedChild;
  final double borderRadius;
  final Object heroTag;

  OpenContainerTransition({this.collapsedChild, this.expandedChild, this.borderRadius = 10.0, this.heroTag});

  @override
  _OpenContainerTransitionState createState() => _OpenContainerTransitionState();
}

class _OpenContainerTransitionState extends State<OpenContainerTransition> {

  void _openExpanded() {
    Navigator.of(context).push(
        PageRouteBuilder(
            pageBuilder: (context, animation, secondaryAnimation) {
              return _ExpandedContainer(child: widget.expandedChild, heroTag: widget.heroTag,);
            },
            transitionsBuilder: (context, animation, secondaryAnimation, child) {
              return FadeTransition(
                opacity: Tween(
                    begin: 0.0,
                    end: 1.0
                ).animate(animation),
                child: child,
              );
            },
            opaque: false
        )
    );
  }

  @override
  Widget build(BuildContext context) {
    Widget finalWidget = Container(
        child: widget.collapsedChild
    );

    if (widget.heroTag != null) {
      finalWidget = Hero(
        tag: widget.heroTag,
        child: Material(
            child: finalWidget
        ),
      );
    }

    return ClipRRect(
      borderRadius: BorderRadius.circular(widget.borderRadius),
      child: GestureDetector(
          onTap: _openExpanded,
          child: finalWidget
      ),
    );
  }
}

class _ExpandedContainer extends StatefulWidget {
  final Widget child;
  final dynamic heroTag;

  _ExpandedContainer({@required this.child, this.heroTag});

  @override
  __ExpandedContainerState createState() => __ExpandedContainerState();
}

class __ExpandedContainerState extends State<_ExpandedContainer> with SingleTickerProviderStateMixin{
  AnimationController controller;
  Animation<double> fadeAnimation;
  Animation<double> offsetAnimation;
  final double dragThreshold = 0.5;

  @override
  void initState() {
    super.initState();

    this.controller = AnimationController(vsync: this, duration: Duration(milliseconds: 200));
    this.fadeAnimation = Tween<double>(begin: 0.8, end: 0.0).animate(this.controller);
    this.offsetAnimation = Tween<double>(begin: 0.0, end: 200.0).animate(this.controller);
  }

  @override
  Widget build(BuildContext context) {
    double height = MediaQuery.of(context).size.height;

    Widget finalWidget = Container(
      child: widget.child,
    );

    if (widget.heroTag != null) {
      finalWidget = Hero(
        tag: widget.heroTag,
        child: Material(
            child: finalWidget
        ),
      );
    }

    return AnimatedBuilder(
      builder: (context, child) {
        return GestureDetector(
            onVerticalDragUpdate: (details) {
              this.controller.value += details.delta.dy / height;
              print(this.controller.value);
            },
            onVerticalDragEnd: (details) {
              this.controller.value > this.dragThreshold ? Navigator.of(context).pop() : this.controller.reverse();
            },
            child: Stack(
              children: <Widget>[
                Center(
                  child: Container(
                    width: double.infinity,
                    height: double.infinity,
                    color: Colors.black.withOpacity(this.fadeAnimation.value),
                  ),
                ),
                Container(
                    transform: Matrix4.translationValues(0.0, this.offsetAnimation.value, 0.0),
                    child: finalWidget
                ),
              ],
            )
        );
      },
      animation: this.controller,
    );
  }
}
